#!/usr/bin/env python
# -*- coding: utf-8 -*-

# by rolfok 2011-06-17
# Version 1.01

# inspiration, info and code from:
# https://code.google.com/p/gears/wiki/GeolocationAPI?redir=1
# http://www.heise.de/netze/news/foren/S-Re-Beispiel-kuerzer/forum-203449/msg-20383299/read/
# http://www.heise.de/netze/meldung/WLAN-MAC-Adressen-Googles-langes-Gedaechtnis-1261893.html

import sys, os, subprocess, getopt
import webbrowser # portable way to open URL in browser

nojson = False
try:
    import simplejson as json # Python < 2.6
except ImportError:
    try:
        import json # Python 2.6 +
    except ImportError:
        nojson = True

class g:
  """Container for global variables."""
  pass

WGET='/usr/bin/wget'      # non-interactive network downloader


def main():

    g.debug = False
    g.google = False
    g.searchch = False
    g.openurl = False
    g.countrycode = False
    g.latlong = False
    macaddr = ''

    showusage = False
    try:
        opts, args = getopt.getopt(sys.argv[1:], "m:dgsloc")
    except getopt.GetoptError:
        showusage = True
    if len(sys.argv) < 2:
        showusage = True

    if showusage == False:
        for opt, arg in opts:
	    if opt == '-m':
	        macaddr = arg
	    if opt == '-d':
	        g.debug = True
	    if opt == '-g':
	        g.google = True
	    if opt == '-s':
	        g.searchch = True
	    if opt == '-l':
	        g.latlong = True
	    if opt == '-o':
	        g.openurl = True
	    if opt == '-c':
	        g.countrycode = True

    if not macaddr: showusage = True
    
    if g.latlong == True or g.countrycode == True:
        g.google = False
        g.searchch = False
        g.openurl = False
        
    if g.latlong == True and g.countrycode == True:
        showusage = True

    if g.searchch == False and g.latlong == False and g.countrycode == False:
        g.google = True # default

    if showusage:
        usage()
        sys.exit(0)

    postdata = '''{
     "version": "1.1.0",
     "host": "maps.google.com",
     "request_address": true,
     "address_language": "de_DE",
      "wifi_towers":
        [{
          "mac_address": "%s"
         }
       ]
    }''' % macaddr

    opt = "-q --post-data='%s' -O -" % postdata
    url = 'http://www.google.com/loc/json'
    execstr = '%s %s -- %s' % (WGET, opt, url)

    err, res = process(execstr)
    if err:
        print res
        sys.exit(1)
  
    if nojson:
        print res
    else:
        enc = json.loads(res)
        if g.debug: print enc
        enc = enc['location']
        if int(enc['accuracy']) > 10000: # meters. usually 166000 gets returned if there was a problem
            print 'accuracy value too high - possible reasons: mac address not found or invalid mac address'
            sys.exit(2)
        else:
            #print enc.keys()
            if g.google:
	        make_url_google((enc['latitude'],enc['longitude']))
	    if g.searchch:
                if enc['address']['country_code'] == 'CH': # CH = Switzerland
                    make_url_searchch((enc['latitude'],enc['longitude']))
            if g.latlong:
	        print enc['latitude'],enc['longitude']
	    if g.countrycode:
	        print enc['address']['country_code']


def open_url(u):
    #webbrowser.open(googleurl) # preferred but does not work on N900 with Opera
    #os.system('opera "%s"' % u)
    os.system('browser --url="%s"' % u)


def make_url_google(latlong):
    # http://mapki.com/wiki/Google_Map_Parameters
    # t= Map Type. The available options are "m" map, "k" satellite, "h" hybrid, "p" terrain.
    # z= Sets the zoom level. 
    # om= Show overview map if set to 1.
    # q= Query - anything passed in this parameter is treated as if it had been typed into the query box on the maps.google.com page.
    # Any text added in parentheses () is displayed in the info window and sidebar in bold. Use + to denote a space.
    googlezoom=15 # adjust this if needed
    #googleurl='http://maps.google.com/maps?t=h&z=%s&om=1&q=%s,%s(%s)' % (googlezoom, latlong[0], latlong[1], 'wlan_geoloc')
    googleurl='http://maps.google.com/maps?nomo=1&ui=maps&t=h&z=%s&om=1&q=%s,%s' % (googlezoom, latlong[0], latlong[1])
    print googleurl
    if g.openurl:
        open_url(googleurl)


def make_url_searchch(latlong):
    searchchzoom=512 # max. 1024
    searchchopt='' # example '&poi=all'
    searchchurl='http://map.search.ch/%s,%s?z=%s%s' % (latlong[0], latlong[1], searchchzoom, searchchopt)
    print searchchurl
    if g.openurl:
        open_url(searchchurl)


def process(s):
    # returns:
    #   if ok: 0, string
    #   if error: errcode, errstring
    proc = subprocess.Popen(s, shell=True, stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    stdout_value, stderr_value = proc.communicate()
    retcode = proc.returncode
    if retcode: # error happened
        return retcode, 'error: %s' % retcode
    else:
        return 0, stdout_value


def usage():
    print '''Usage: %s -m macaddr
optional arguments:
-g     print google url (default)
-s     print search.ch url, if possible, otherwise print nothing
-o     with -g or -s, print url and open it in the browser
-l     print just latitude longitude (turns above options off)
-c     print country code (turns above options off)
-d     debug
example:
%s -m 00:11:24:EC:72:CF
''' % (sys.argv[0], sys.argv[0])


main()
